function [trend_ETs, C0_values] = LF_identify(M, omega0, C0, maxET, C0fig)
%
%  Identify trend ETs using LowFreq method.
%
%  Parameters:
%    M -- matrix with singular vectors in columns
%    omega0 -- upper boundary of low frequencies interval [0, omega0]
%    C0 -- threshold of LowFreq method
%    maxET -- amount of first examined ETs
%    C0fig -- a handle of figure for plotting sorted values of C0
%             calculated for all ETs. It's optional, if isn't specified 
%             than this won't be plotted.
%
%  Returns:
%    trend_ETs -- numbers of identificated ETs (starting from 1)
%    C0_values -- values of C (LF criterion) for each identificated ET
%
%  (c) Theodore Alexandrov (autossa@gmail.com)
%  Last modified: 4 May 2006  
%

L=size(M, 1);
rankM=min(L, size(M, 2));

if ~exist('maxET','var')
   maxET=L;
end

maxET = min(maxET, rankM);
    

C0_values=zeros(maxET,1);
trend_ETs=[];

j=1;
for i=1:maxET
    C0_values(i) = LFvalue_vect(M, omega0, i);
    if  C0_values(i) > C0
        trend_ETs(j) = i;
        j=j+1;
    end
end


if exist('C0fig','var') && ~isempty(C0fig)
   figure(C0fig);
   
   sorted_C0s = sort(C0_values);
%   fprintf(1, 'First 15 C0 values: \n');
%   for i=L:-1:L-10
%      fprintf(1, '   %3.3d\n', sorted_C0s(i));
%   end
   
   plot(sorted_C0s, '-o');
   legend('Sorted C0 values');
end

