function LF_value = LFvalue_vect(M, omega0, ET)
%
%  Calculate LowFreq value for the given ET
%
%  Parameters:
%    M -- a matrix with singular vectors in columns
%    omega0 -- upper boundary of low frequencies interval [0, omega0]
%    ET -- number of eigen triple to examine (numbers starts with 1)
%
%
%  Comments:
%    1) Since sum of periodogram values (on all frequencies) is equal to 
%    the squared norm of the examined vector (time series), we will 
%    normalize not sum of periodogram values but the vector (time series)
%    itself beforehand, i.e. instead of
%        LF_value = \sum_{w is LF} \Pi^F(w)  /  \sum_{all w} \Pi(w)
%    we use
%       1) F -> G=F/norm(F)
%       2) LF_value = \sum_{w is LF} \Pi^G(w)
%
%    2) We define LowFreq value of zero vector equal to 0.
%
%  (c) Theodore Alexandrov (autossa@gmail.com)
%  Last modified: 27 November 2005 
%


L=size(M,1);

vect = M(:,ET);
nrm = norm(vect);
% see comments above
if nrm==0
   LF_value=NaN;
   return;
end
vect = vect/nrm;

LF_value=0;

% calculate sum of periodogram values for low frequencies
k=1;
w=0; 
while w<omega0
    LF_value = LF_value + Periodogram(vect,k);
    k=k+1;
    w=(k-1)/L;
end

