function [per, perETs, rho0val, MSEval_atrho0, ...
          ETs, rho_atETs, period_atETs, T2ETs, rhoT2_atETs, ...
          rhos, MSEvals]    =     autoperiodicity(ts, L, MSEthresh, ...
                                                  maxET, ...
                                                  rho0left, rho0right, rho0step, ...
                                                  Tlimmin, Tlimmax, ...
                                                  showprogress)
                                              
%
%   Extracts periodical component of given time series
%
%   All parameters are mandatory!
%
%   Parameters: 
%	ts -- a column, containing time series values
%	L -- window width (1<L<time series length), 
%	     usually L=floor(N/2)
%	MSEthresh -- 
%	maxET -- amount of first ETs to be examined, usually floor(L/2)
%	rho0left -- where search of the best rho0 starts, usually 0.5
%	rho0right -- where it finishes, usually 1
%	rho0step -- step of the search, usually 0.1
%	Tlimmin,Tlimmax -- specify interval for period, for monthly data 11 and 13
%	showprogress 
%
%
%   Returns:
%	per -- an extracted periodic component
%	perETs -- eigentriples which correspond to the extracted component
%	rho0val -- value of rho0
%	MSEval_atrho0 -- value of the criterion achieved at the selected rho0val
%	ETs --
%	rho_atETs --
%	period_atETs --
%	T2ETs --
%	rhoT2_atETs --
%	rhos --
%	MSEvals -- values of the criterion calculated at rhos
%
%
%   Theodore Alexandrov (autossa@gmail.com)
%   Last modified: Jun 19, 2006
%


rho0s_amount=round((rho0right-rho0left)/rho0step)+1;
rhos=[rho0left:rho0step:rho0right];

N=length(ts);

[sing_values,U,V] = ssa(ts, L);

d=min(length(sing_values), maxET);
ETs=[1:d-1];
T2ETs=[1:d];

if d<2
    per=zeros(size(ts));
    perETs=[];
    rho0val=[];
    MSEval_atrho0=[];
    ETs=[];
    rho_atETs=[];
    period_atETs=[];
    T2ETs=[];
    rhoT2_atETs=[];
    rhos=[];
    MSEvals=[];
    
    return;
end
[rho_atETs, rhoT2_atETs] = Fourier2_values(U, ETs, T2ETs, showprogress); 

period_atETs = period_est(U, ETs, showprogress);

%look over rho0 values
MSEvals=zeros(rho0s_amount,1);
if showprogress
    progressbar = waitbar(0, 'Searching for the optimal rho0...');
end
rho0_num=rho0s_amount;
rho0=rhos(end);
new_ETs = Fourier_identify_per(ETs, rho_atETs, period_atETs, T2ETs, rhoT2_atETs, rho0, Tlimmin, Tlimmax);
allid_ETs=new_ETs;
result = ssa_reconstruct(sing_values, U, V, new_ETs);
MSEvals(rho0_num) = sum(result.*result)/N;% / sqrt(sum( (ts-mean(ts)).*(ts-mean(ts)) ));
for rho0=rhos(end:-1:1)
    tmp_ETs = Fourier_identify_per(ETs, rho_atETs, period_atETs, T2ETs, rhoT2_atETs, rho0, Tlimmin, Tlimmax);
    
    new_ETs=setdiff(tmp_ETs, allid_ETs);
    
    allid_ETs=tmp_ETs;

    if length(new_ETs)==0
        MSEvals(rho0_num)=0;
    else
        result = ssa_reconstruct(sing_values, U, V, new_ETs);
        MSEvals(rho0_num) = sum(result.*result)/N;% / sqrt(sum( (ts-mean(ts)).*(ts-mean(ts)) ));
    end
    
    if showprogress
        waitbar((rho0-rhos(1))/(rhos(end)-rhos(1)), progressbar);
    end

    rho0_num = rho0_num - 1;
end %rho0 

rho0_num=1;
for rho0=rhos
    if MSEvals(rho0_num) >= MSEthresh
        fl_foundrho0=1;
        MSEval_atrho0 = MSEvals(rho0_num);
        rho0val = rho0;
        
        perETs = Fourier_identify_per(ETs, rho_atETs, period_atETs, T2ETs, rhoT2_atETs, rho0val, Tlimmin, Tlimmax);
        perETs = unique(perETs);
        per = ssa_reconstruct(sing_values, U, V, perETs);
        
        if showprogress
            waitbar(1, progressbar);
            close(progressbar);
        end

        return;
    end
    
    rho0_num = rho0_num + 1;
end

MSEval_atrho0 = MSEvals(end);
rho0val = rhos(end);
        
perETs = Fourier_identify_per(ETs, rho_atETs, period_atETs, T2ETs, rhoT2_atETs, rho0val, Tlimmin, Tlimmax);
perETs = unique(perETs);
per = ssa_reconstruct(sing_values, U, V, perETs);

if showprogress
    waitbar(1, progressbar);
    close(progressbar);
end