function varargout = gautoperiodicity(varargin)
% GAUTOPERIODICITY M-file for gautoperiodicity.fig

%
%   Extracts periodical component of given time series
%
%   Usage:
%	gautoperiodicity(ts, L, MSEthresh, maxET, rho0left, rho0right, rho0step, Tlimmin, Tlimmax)
%	gautoperiodicity(ts, L)
%
%   Mandatory parameters: 
%	ts -- a column, containing time series values
%	L -- window width (1<L<time series length), 
%	     usually L=floor(N/2)
%
%   Optional parameters:
%	MSEthresh -- 
%	maxET -- amount of first ETs to be examined, usually floor(L/2)
%	rho0left -- where search of the best rho0 starts, usually 0.5
%	rho0right -- where it finishes, usually 1
%	rho0step -- step of the search, usually 0.1
%	Tlimmin,Tlimmax -- specify interval for period, for monthly data 11 and 13
%
%
%   Theodore Alexandrov (autossa@gmail.com)
%

% Last Modified by GUIDE v2.5 23-Jun-2006 15:49:35

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @gautoperiodicity_OpeningFcn, ...
                   'gui_OutputFcn',  @gautoperiodicity_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before gautoperiodicity is made visible.
function gautoperiodicity_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to gautoperiodicity (see VARARGIN)

% Choose default command line output for gautoperiodicity
handles.output = hObject;


if length(varargin)<2
    uiwait( errordlg('You must specify time series and L', 'Error') );
end

% mandatory parameters
if length(varargin)>=1
   handles.givendata.ts = varargin{1};
   handles.data.ts = handles.givendata.ts;
end
if length(varargin)>=2
   handles.givendata.L = varargin{2};
   handles.data.L = handles.givendata.L;
end


% optional parameters
if length(varargin)>=3  
   handles.givendata.MSEthresh = varargin{3};
   handles.data.MSEthresh = handles.givendata.MSEthresh;
else
   handles.data.MSEthresh = 0.05;
end
if length(varargin)>=4  
   handles.givendata.maxET = varargin{4};
   handles.data.maxET = handles.givendata.maxET;
else
   handles.data.maxET = floor(handles.data.L/2);
end
if length(varargin)>=5
   handles.givendata.rho0left = varargin{5};
   handles.data.rho0left = handles.givendata.rho0left;
else
   handles.data.rho0left = 0.7; %default value
end
if length(varargin)>=6
   handles.givendata.rho0right = varargin{6};
   handles.data.rho0right = handles.givendata.rho0right; 
else
   handles.data.rho0right = 1; % default value
end
if length(varargin)>=7
   handles.givendata.rho0step = varargin{7};
   handles.data.rho0step = handles.givendata.rho0step;
else
   handles.data.rho0step = 0.02; % default value
end
if length(varargin)>=8
   handles.givendata.Tlimmin = varargin{8};
   handles.data.Tlimmin = handles.givendata.Tlimmin;
else
   handles.data.Tlimmin = 11; % default value
end
if length(varargin)>=9
   handles.givendata.Tlimmax = varargin{9};
   handles.data.Tlimmax = handles.givendata.Tlimmax;
else
   handles.data.Tlimmax = 13; % default value
end

handles.data.omega0FORtrresid = 0.07;

handles.settings.resuts_filename = 'autoperiodicity.res';

% clear all graphs, plot ts and clear all made changes in settings
reset_all(hObject, handles);

% Update handles structure
guidata(hObject, handles);



% --- clear all graphs, plot ts and clear all made changes in settings
function reset_all(hObject, handles);

set(handles.L_input, 'String', num2str(handles.data.L));
set(handles.maxET_input, 'String', num2str(handles.data.maxET));
set(handles.MSEthresh_input, 'String', num2str(handles.data.MSEthresh));
set(handles.rho0left_input, 'String', num2str(handles.data.rho0left));
set(handles.rho0right_input, 'String', num2str(handles.data.rho0right));
set(handles.rho0step_input, 'String', num2str(handles.data.rho0step));
set(handles.Tlimmin_input, 'String', num2str(handles.data.Tlimmin));
set(handles.Tlimmax_input, 'String', num2str(handles.data.Tlimmax));

set(handles.omega0FORtrresid_input, 'String', num2str(handles.data.omega0FORtrresid));

clear_all_graphs(handles);
plot_ts(handles);


% Update handles structure
guidata(hObject, handles);




% --- clear all graphs
function clear_all_graphs(handles);

axes(handles.ts_axes);
cla;
reset(gca);
box on;

axes(handles.periodicity_axes);
cla;
reset(gca);
box on;

axes(handles.MSE_axes);
cla;
reset(gca);
box on;

axes(handles.rhovalues_axes);
cla;
reset(gca);
box on;



% --- plot handles.data.ts
function plot_ts(handles);

% plot time series 
axes(handles.ts_axes)
plot(handles.data.ts);
xlim([1 length(handles.data.ts)])
grid on;
set(gca, 'XMinorGrid', 'on');





% --- Outputs from this function are returned to the command line.
function varargout = gautoperiodicity_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;



function L_input_Callback(hObject, eventdata, handles)
% hObject    handle to L_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of L_input as text
%        str2double(get(hObject,'String')) returns contents of L_input as a double

handles.data.L = str2num(get(hObject, 'String'));

if handles.data.maxET > handles.data.L
    errordlg(['Maximal ET (', num2str(handles.data.maxET), ') < L (', num2str(handles.data.L) ,'). Input correct maximal ET number!']);
end

% Update handles structure
guidata(hObject, handles);




% --- Executes during object creation, after setting all properties.
function L_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to L_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end




function rho0left_input_Callback(hObject, eventdata, handles)
% hObject    handle to rho0left_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of rho0left_input as text
%        str2double(get(hObject,'String')) returns contents of rho0left_input as a double
handles.data.rho0left = str2double(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function rho0left_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to rho0left_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function rho0right_input_Callback(hObject, eventdata, handles)
% hObject    handle to rho0right_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of rho0right_input as text
%        str2double(get(hObject,'String')) returns contents of rho0right_input as a double

handles.data.rho0right = str2double(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);



% --- Executes during object creation, after setting all properties.
function rho0right_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to rho0right_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function rho0step_input_Callback(hObject, eventdata, handles)
% hObject    handle to rho0step_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of rho0step_input as text
%        str2double(get(hObject,'String')) returns contents of rho0step_input as a double

handles.data.rho0step = str2double(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);



% --- Executes during object creation, after setting all properties.
function rho0step_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to rho0step_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function MSEthresh_input_Callback(hObject, eventdata, handles)
% hObject    handle to MSEthresh_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of MSEthresh_input as text
%        str2double(get(hObject,'String')) returns contents of MSEthresh_input as a double

handles.data.MSEthresh = str2double(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function MSEthresh_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to MSEthresh_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function Tlimmin_input_Callback(hObject, eventdata, handles)
% hObject    handle to Tlimmin_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of Tlimmin_input as text
%        str2double(get(hObject,'String')) returns contents of Tlimmin_input as a double

handles.data.Tlimmin = str2double(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function Tlimmin_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to Tlimmin_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function Tlimmax_input_Callback(hObject, eventdata, handles)
% hObject    handle to Tlimmax_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of Tlimmax_input as text
%        str2double(get(hObject,'String')) returns contents of Tlimmax_input as a double

handles.data.Tlimmax = str2double(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);



% --- Executes during object creation, after setting all properties.
function Tlimmax_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to Tlimmax_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function maxET_input_Callback(hObject, eventdata, handles)
% hObject    handle to maxET_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of maxET_input as text
%        str2double(get(hObject,'String')) returns contents of maxET_input as a double

handles.data.maxET = str2num(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function maxET_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to maxET_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function omega0FORtrresid_input_Callback(hObject, eventdata, handles)
% hObject    handle to omega0FORtrresid_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of omega0FORtrresid_input as text
%        str2double(get(hObject,'String')) returns contents of omega0FORtrresid_input as a double

handles.data.omega0FORtrresid = str2num(get(hObject, 'String'));

% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function omega0FORtrresid_input_CreateFcn(hObject, eventdata, handles)
% hObject    handle to omega0FORtrresid_input (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end





% --- Executes on button press in recalculate.
function recalculate_Callback(hObject, eventdata, handles)
% hObject    handle to recalculate (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% get the structure in the subfunction

if handles.data.maxET > handles.data.L
    errordlg(['Maximal ET (', num2str(handles.data.maxET), ') < L (', num2str(handles.data.L) ,'). Input correct maximal ET number!']);
    return;
end

ts = handles.data.ts;
L = handles.data.L;
MSEthresh = handles.data.MSEthresh;

if ~exist('ts', 'var') || ~exist('L', 'var') || ~exist('MSEthresh', 'var')
    uiwait( errordlg('Time series or L or MSEthresh wasn''t specified', 'Error', 'on') );
    return;
end

maxET = handles.data.maxET;
rho0left = handles.data.rho0left;
rho0right = handles.data.rho0right;
rho0step = handles.data.rho0step;
Tlimmin = handles.data.Tlimmin;
Tlimmax = handles.data.Tlimmax;


showprogress=1;

[per, perETs, rho0, MSEval_atrho0, ...
          ETs, rho_atETs, period_atETs, T2ETs, rhoT2_atETs, ...
          rhos, MSEvals] = ...
        autoperiodicity(ts, L, MSEthresh, ...
                        maxET, ...
                        rho0left, rho0right, rho0step, ...
                        Tlimmin, Tlimmax, ...
                        showprogress);
                    
handles.results.per = per;
handles.results.perETs = perETs;
handles.results.rho0 = rho0;

% plot time series and trend 
axes(handles.ts_axes)
plot(ts);
hold on;
plot(ts-per, 'r');
hold off;

legend('Original', 'Adjusted');
xlim([1 length(ts)])
grid on;
set(gca, 'XMinorGrid', 'on');

% plot residual time series (original - trend)
axes(handles.periodicity_axes)
plot(per);
legend(['Periodicity, rho0=', num2str(rho0), ', ETs: ', ETsnum2str(perETs)]);
xlim([1 length(ts)])
grid on;
set(gca, 'XMinorGrid', 'on');


axes(handles.MSE_axes)
%plot(rhos, (MSEvals - min(MSEvals)) / ( max(MSEvals)-min(MSEvals)));
plot(rhos, MSEvals);
hold on
plot(rhos, ones(size(rhos))*MSEthresh, 'r');
legend(['||per||^2, \rho_0: MSE(', num2str(rho0), ') = ', num2str(MSEval_atrho0) ], ...
       ['MSE thresh = ', num2str(MSEthresh)], 'Location', 'NorthWest');
if length(rhos)>1
    xlim([rhos(1) rhos(end)])
end
grid on;
set(gca, 'XMinorGrid', 'off');
hold off;


axes(handles.rhovalues_axes)
if length(ETs)>0 & length(rho_atETs)>0 % against time series of rank 1
    barh=bar(ETs, rho_atETs);
end
hold on
plot(ETs, ones(size(ETs))*rho0, 'r');
legend('rho values for (ET, ET+1)', 'Location', 'SouthEast');
if length(ETs)>1
    xlim([1 ETs(end)])
end
grid on;
set(gca, 'XMinorGrid', 'off');
hold off;


% Update handles structure
guidata(hObject, handles);




% --- Executes on button press in scrshot_all_btn.
function scrshot_all_btn_Callback(hObject, eventdata, handles)
% hObject    handle to scrshot_all_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if ~isfield(handles, 'operations') || ~isfield(handles.operations, 'screenshots')
    handles.operations.screenshots=1;
else
    handles.operations.screenshots=handles.operations.screenshots+1;
end

frame=getframe(gcf);
img=frame2im(frame);
scrfilename=['screenshot', num2str(handles.operations.screenshots), '.bmp'];
imwrite(img,scrfilename, 'bmp')

helpdlg(['Screenshot was saved to "', scrfilename, '"']);

% Update handles structure
guidata(hObject, handles);




% --- Executes on button press in scrshot_trresid_btn.
function scrshot_trresid_btn_Callback(hObject, eventdata, handles)
% hObject    handle to scrshot_trresid_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if ~isfield(handles, 'operations') || ~isfield(handles.operations, 'screenshots')
    handles.operations.screenshots=1;
else
    handles.operations.screenshots=handles.operations.screenshots+1;
end

y0=218;
x0=1;
height=700;
width=980;
frame=getframe(gcf, [x0,y0,width-x0,height-y0]);
img=frame2im(frame);
scrfilename=['screenshot', num2str(handles.operations.screenshots), '.bmp'];
imwrite(img,scrfilename, 'bmp')

helpdlg(['Screenshot was saved to "', scrfilename, '"']);

% Update handles structure
guidata(hObject, handles);



% --- Executes on button press in saveresults_btn.
function saveresults_btn_Callback(hObject, eventdata, handles)
% hObject    handle to saveresults_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if ~isfield(handles, 'results')
    errordlg('No results', 'Error');
    return;
end
    
save_autoperiodicity(handles.settings.resuts_filename, ...
                    handles.data.ts, ...
                    handles.data.L, ...
                    handles.data.MSEthresh, ...
                    handles.data.maxET, ...
                    handles.data.rho0left, ...
                    handles.data.rho0right, ...
                    handles.data.rho0step, ...
                    handles.data.Tlimmin, ...
                    handles.data.Tlimmax, ...
                    ...
                    handles.results.per, ...
                    handles.results.perETs, ...
                    handles.results.rho0);

helpdlg(['The results were saved to "', handles.settings.resuts_filename, '"']);









% --- Executes on button press in trOFresid_btn.
function trOFresid_btn_Callback(hObject, eventdata, handles)
% hObject    handle to trOFresid_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if isfield(handles, 'results') && isfield(handles.results, 'per')
    gautotrend(handles.data.ts - handles.results.per, handles.data.L, handles.data.omega0FORtrresid);
else
    errordlg('No results. Extract periodicity first.', 'Error');
end



% --- Executes on button press in trOFts_btn.
function trOFts_btn_Callback(hObject, eventdata, handles)
% hObject    handle to trOFts_btn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

gautotrend(handles.data.ts, handles.data.L, handles.data.omega0FORtrresid);

