function ts = loadts_txt(datapath, TS_COLUMN, HEADER_LINES, COMMENTSTYLE)
%
%  Loads column from a given data file 
%
%  Mandatory parameters:
%    datapath -- full path to a data file
%
%  Optional parameters:
%    TS_COLUMN -- number of a column containing data, starting with 1 (default 1)
%    HEADER_LINES -- how many first lines must be skipped (default 0)
%    COMMENTSTYLE -- optional parameter, see comment style of Matlab's textread (default 'shell' <-> '#')
%
%  Returns:
%    ts -- loaded time series (a column)
%
%  (c) Theodore Alexandrov (autossa@gmail.com)
%  Last modified: 24 Sep 2006  
%


if ~exist('TS_COLUMN','var')
	TS_COLUMN=1;
end

if ~exist('HEADER_LINES','var')
	HEADER_LINES=0;
end

%default comment style is '#' at the beginning of line
if ~exist('COMMENTSTYLE','var')
	COMMENTSTYLE='shell';
end



data = textread(datapath, '', 'headerlines', HEADER_LINES, 'commentstyle', COMMENTSTYLE, 'emptyvalue', NaN);
% remove rows containing NaN which are put on the positions of empty values
data(any(isnan(data)'),:) = [];

ts=data(:,TS_COLUMN);

