function Frec = ssa_reconstruct(sing_values, U, V, ETs)
%
%  Reconstruct time series by given ETs components.
%
%  Parameters:
%       sing_values -- array of singular values 
%       U -- matrix of eigenvectors
%       V -- matrix of factorvectors
%       ETs -- array with numbers of eigen triples (ETs) which
%              should be used at reconstruction (starting from 1)
%
%  Returns:
%       Frec -- a time series reconstructed by given ETs
%               (a column)
%
%
%  If you modify this source code somehow, please send me the modified version.
%
%
%  (c) Theodore Alexandrov (autossa@gmail.com)
%  Last modified: 8 May 2006 
%

L=size(U,1);
K=size(V,1);
N=K+L-1;

% required traj.matrix
X=zeros(L,K);

for i=1:length(ETs)
    % form elementary traj.matrix for each given ET
    ET=ETs(i);
    
    sing_value=sing_values(ET);
    u=U(1:L, ET);
    v=V(1:K, ET);
    
    Xi=sing_value*u*v';
    
    % add this elementary traj.matrix to the traj.matrix 
    % which will be used later for reconstruction
    X = X + Xi;
end


Frec=zeros(N,1); % reconstructed time series

% 
% diagonal averaging
%
L0 = min(L,K);
K0 = max(L,K);

if L<K 
    Y=X;
else
    Y=X';
end

for k=1:L0-1
    d=0;
    for m=1:k
        d = d + Y(m,k-m+1);
    end
    Frec(k) = 1/k * d;
end

for k=L0:K0
    d=0;
    for m=1:L0
        d = d + Y(m,k-m+1);
    end
    Frec(k) = 1/L0 * d;
end

for k=K0+1:N
    d=0;
    for m=k-K0+1:N-K0+1
        d = d + Y(m,k-m+1);
    end
    Frec(k) = 1/(N-k+1) * d;
end


